@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2014 -- leonerd@leonerd.org.uk
 
 package IO::Async::ChildManager;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.63';
+our $VERSION = '0.65';
 
 # Not a notifier
 
@@ -19,14 +19,10 @@ use IO::Async::OS;
 use Carp;
 use Scalar::Util qw( weaken );
 
-use POSIX qw( _exit sysconf _SC_OPEN_MAX dup dup2 nice );
+use POSIX qw( _exit dup dup2 nice );
 
 use constant LENGTH_OF_I => length( pack( "I", 0 ) );
 
-# Win32 [and maybe other places] don't have an _SC_OPEN_MAX. About the best we
-# can do really is just make up some largeish number and hope for the best.
-use constant OPEN_MAX_FD => eval { sysconf(_SC_OPEN_MAX) } || 1024;
-
 =head1 NAME
 
 C<IO::Async::ChildManager> - facilitates the execution of child processes
@@ -364,6 +360,16 @@ Shortcuts for C<fd0>, C<fd1> and C<fd2> respectively.
 
 A reference to a hash to set as the child process's environment.
 
+Note that this will entirely set a new environment, completely replacing the
+existing one. If you want to simply add new keys or change the values of some
+keys without removing the other existing ones, you can simply copy C<%ENV>
+into the hash before setting new keys:
+
+ env => {
+    %ENV,
+    ANOTHER => "key here",
+ }
+
 =item nice => INT
 
 Change the child process's scheduling priority using C<POSIX::nice>.
@@ -604,7 +610,7 @@ sub _spawn_in_child
          }
       }
 
-      foreach ( 0 .. OPEN_MAX_FD ) {
+      foreach ( IO::Async::OS->potentially_open_fds ) {
          next if $fds_refcount{$_};
          next if $_ == fileno $writepipe;
          POSIX::close( $_ );
